# R program - Digital_Cyclist_0_2
# developed by J.H.G.M. van Beek - May 2008 and December 2010, updated December 2013

#  This program simulates energy conversion and heat transport in a cyclist, in particular during a mountain time trial
#     Copyright (C) <2013>  <J.H.G.M. van Beek>
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

# THIS PROGRAM IS NOT SUITABLE TO SUPPORT ANY DECISIONS THAT MAY AFFECT THE HEALTH OR WELL-BEING OF A PERSON

# this is an updated version of the program used to calculate the simulations of a cyclist published in
# van Beek JHGM, Supandi F, Gavai AK, Binsl TW, and Hettling H.
# Simulating the physiology of athletes during endurance sports events: modelling human energy conversion and metabolism.
# Phil Trans Roy Soc A 369: 4295-4315, 2011
# Please refer to this publication in appropriate cases.

# this model incorporates equations from the Stolwijk model of human heat transport
# as described in "Dynamic Mathematical Model of Physiological Regulation of Body Temperature
# in human beings", NASA, author: Stolwijk, Publication Date August 1, 1971
# The equations from the Stolwijk model are implemented here in R.
#
# The file layout was made in software text editor ConTEXT 0.98.5 (http://www.context.cx; freeware)
#
# An animated plot is produced by gradually adding points to a graph
#
# Heat rates and fluxes given in kcal/hr.
# 1 cal (th) = 4.184 Joule; 1 kcal (th) = 4184 Joule
# 1 kcal/hr = 4184/3600  = 1.1622 Watt.
# rest metabolism 74.4 kcal/hr = 86.47 Watt

# the energy usage of a cyclist is based on
# T.S. Olds et al. Mathematical model of cycling performance. J.Appl.Physiol. 75:730-737, 1993.
#

# the next statements clean the R workspace and load required libraries
rm(list=ls())
library(deSolve) # in earlier versions of the program the R package 'odesolve' was used which is presently deprecated
library(xlsx)    # package to write data to Excel files
cat("\nR workspace cleaned\n\n")

# a time stamp is created which is added to the output filename for unique identification
date_aux<-gsub("\\s", "-",  format(Sys.time(), "%a %b%d %H.%M %Y"))

#if the output to screen should be saved: uncomment the next sink statement by removing the # at the very first position of the line
# sink(file=paste("sinkfile ",date_aux,".txt"),split=TRUE)
# output to screen is written to a file in the R work directory

# READING PARAMETERS - READ CONSTANTS FOR CONTROLLED THERMAL SYSTEM
source("Thermal_Parameters_Digital_Cyclist.R")

# READING REST OF PARAMETERS
source("Rest_Parameters.R")

# ADD TIMING AND PLOT PARAMETERS TO THE PARAMETER ARRAY 'PA'
source("Timing_and_Plot_Parameters.R")

# Read in functions which can make parameters during run variable
source("Switch_functions.R")

# READ FUNCTIONS THAT DESCRIBE SLOPE AND HEIGHT OF THE TRAJECTORY
source("Slope_Height_Trajectory.R")


# FF is the rate of change of temperature. It is not used in the present program and therefore set to zero.
    FF <- rep(0, 25)

    finish_time_flag <- 0.
    raw_finish_time <- 0.
    raw_finish_distance <- 0.
    raw_finish_speed <- 0.

# MODEL FUNCTION DECLARED IN NEXT STATEMENT USED BY THE INTEGRATING ROUTINE 'lsoda'
# function 'model' is used to calculate the time derivatives of temperatures in the model.
# The time derivatives are additions of heat fluxes formed by heat production and by transport terms which
# are divided by the heat capacity of the compartment.
# In addition in this function some results are calculated to add to the 'results' data structure
# which is generated by the integrating routine 'lsoda'.
# At each time point this function is needed by routine 'deSolve' to integrate the system of
# ordinary differential equations which constitutes the model.
# As a side effect this function 'model' adds a point to the plot at time points requested.
# t is the simulation time
# y is the vector of compartment temperatures
# pa is the concatenated parameter array for the temperature model

# THE MODEL FUNCTION TO BE INTEGRATED IS DEFINED
# MODEL FUNCTION - MODEL FUNCTION - MODEL FUNCTION - MODEL FUNCTION - MODEL FUNCTION - MODEL FUNCTION

model <- function(t, y, pa) {
    # ydot, time derivative of temperature
    # ydot[1], in first compartment (core of head, brain) with index 1.
    # Reference to array members is usually by name.

    # calculate the altitude (height) during the climb from the position on the road at time t
    hAH <- Height_Trajectory(t,y,pa)

    # calculate the barometric pressure at that height
    if (pa["pb_red_by_alt"] == 1) {
    Pbvar <- pa["PB"] * (1 - 0.0000225577 * hAH)^5.25588
    }
    else Pbvar <- pa["PB"]

    #uncomment next statement (i.e. remove '#' before the statement) if you want to monitor function calls and progress
    #cat("\ntime ",t," height = ",hAH," Pbar = ",Pbvar)

    # next we calculate the muscular power (work per unit time)
    # available from the muscles of the athlete to sustain velocity against gravity and friction or to accelerate
    # it is assumed that 1 ml of O2 (STPD) corresponds to 20.9 J/ml, corresponding to a respiratory quotient
    # of 0.96 (Di Prampero, 1979)
    # 1 kcal (th) = 4184 Joule
    # 1000 ml/min = 20,900 J/min = 4.99522 kcal/min = 299.713 kcal/hr

    # at t<0 WORK is calculated from VO2init i.e. the initial rate of oxygen consumption
    # the amount of WORK above basal metabolic rate of 74.4 kcal/hr may be due to muscle tension,
    # increased physiological activity of digestive system etc.
    # of this internal work nothing is used for external work Pext (Watt)
    # Pext equals WR in the Olds, 1993 paper
      if (t<=0.) {
         # converting VO2init in litre O2 / min to WORK in kcal/hr
         WORK <- pa["VO2init"]* 299.713
         # calculating WORKI from oxygen uptake measurements before starting to cycle
         WORKI <- WORK - pa["BASAL_METABOLIC_RATE"]
         Pext <- 0.
         }
    # during exercise the work rate is set to a certain fraction of VO2max which can be sustained
    # for the duration of the exercise period.
    # see f factor in manuscripts by Olds, e.g. Fig. 1 in 1993 manuscript
      if (t>0.) {
         # nVO2 (normalized VO2max) =  0.4397485 + 0.0007373 * barometric pressure (mmHg)
         # here the barometric pressure calculated for that altitude is used : pa["Pbvar"]
         if (pa["VO2_red_by_alt"] == 1) {
         nVO2 <-  0.4397485 + 0.0007373 * Pbvar
         }
         else nVO2 <- 1
         # uncomment next cat statement if you wnat to see Pbar and nVO2 printed out during each function call
         # cat("\ntime = ",t," Pbar = ",Pbvar," nVO2 = ",nVO2)
         # what is the maximum part of VO2max that can be used for a certain period of time
         partVO2 <- nVO2 * pa["brainpush"]* pa["VO2max"]
         # what is the energy turnover in kcal/hr that is equivalent to partVO2
         WORKEQ <- partVO2*299.713
         WORK <- WORKEQ
         # increase in energy turnover over basal metabolic rate in kcal/hr
         WORKEQI <- WORKEQ - pa["BASAL_METABOLIC_RATE"]
         # the part of increase energy turnover available for mechanical work done by the athlete
         Pext <- WORKEQI * pa["DeltaMechEff"]/0.8604201 # in Watt, calculated from Delta mechanical efficiency
         # WORKI is the heat generation above baseline level associated with the muscle work performed
         WORKI <-  WORKEQI - Pext* 0.8604201 # converting 1 Watt to 0.8604201 kcal/hr
         }

    ydot <- matrix(0,nrow=27,ncol=1)

    # switches in experimental parameters are made by the following function
    T_air <- Tair_switch(t)

    # phase = 0 before cycling, phase = 1 when cycling
    phase <- Phase_switch(t)

    # v_cycle in m/s is calculated from y[27] in m / hr ; v_wind, frontal wind speed in m/s
    v_cycle <- y[27]/3600.
    v_wind <- pa["V"]


    # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION
    # set experimental conditions

   # combined heat transfer coefficient radiative (HR) plus convective (HC)
   H <- numeric(length = 6)
   for (i in 1:6) {
      H[i] <- (HR[i] + HC[i] * ( (v_wind + v_cycle) / 0.1)^0.5) * S[i]
      }

    ii <- floor(T_air/5)
    P_air <- pa["RH"] * (P[ii] + (P[ii+1] - P[ii])*(T_air-5*ii)/5.0)


 # construct vectors which are going to be filled in iterative loops
     error <- numeric(length = 25)
     cold <- numeric(length = 25)
     warm <- numeric(length = 25)
     qq <- numeric(length = 24)
     bf <- numeric(length = 24)
     e <- numeric(length = 24)
     pskin <- numeric(length = 6)
     emax <- numeric(length = 6)
     bc <- numeric(length = 24)
     td <- numeric(length = 24)
     hf <- numeric(length = 25)
     # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION


     for (n in 1:25) {
     if (FF[n] >= 0.) FF[n] <<- 0.
     # note that RATE is always zero in the present model
     error[n] <- y[n] - TSET[n] + RATE[n]*FF[n]
       # in contrast to the Stolwijk report (1971) cold[n] is set to -error[n]
       # this seems in line with the use of cold[n] in the control equations for sweat, dilat, stric, chill
     if(error[n]<0.) { cold[n] <- -error[n]; warm[n] <- 0. }
     if(error[n]==0.) {cold[n]<- 0.; warm[n] <- 0.}
     if(error[n]>0.) {warm[n] <- error[n]; cold[n]<-0. }
     }

     warms <- 0.
     colds <- 0.
     for (i in 1:6) {
       k <- 4*i
       warms <- warms + warm[k]*SKINR[i]
       colds <- colds + cold[k]*SKINR[i]
     }

     sweat <- pa["CSW"]*error[1] + pa["SSW"]*(warms - colds) + pa["PSW"]*warm[1]*warms
     dilat <- pa["CDIL"]*error[1]+ pa["SDIL"]*(warms - colds) + pa["PDIL"]*warm[1]*warms
     stric <- -pa["CCON"]*error[1] - pa["SCON"]*(warms - colds) + pa["PCON"]*cold[1]*colds
     chill <- -pa["CCHIL"]*error[1] - pa["SCHIL"]*(warms - colds) + pa["PCHIL"]*cold[1]*colds

     # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION

     if (sweat < 0.) sweat <- 0.
     if (dilat < 0.) dilat <- 0.
     if (stric < 0.) stric <- 0.
     if (chill < 0.) chill <- 0.

     for (i in 1:6) {
       n <- 4*i - 3

       # heat production qq : we use qq instead of q because the latter may have a special meaning in R
       qq[n] <- QB[n]
       # blood flow bf
       bf[n] <- BFB[n]
       # evaporation e
       e[n] <- EB[n]
       # heat production associated with muscle work and with shivering (chill) are added to the basal heat rate
       qq[n+1] <- QB[n+1] + WORKM[i]*WORKI + CHILM[i]*chill
       e[n+1] <- 0.0
       bf[n+1] <- BFB[n+1] + qq[n+1] - QB[n+1]
       qq[n+2] <- QB[n+2]
       e[n+2] <- 0.
       bf[n+2] <- BFB[n+2]
       qq[n+3] <- QB[n+3]
       e[n+3] <- (EB[n+3] + SKINS[i]*sweat)*2.^(error[n+3]/pa["BULL"])
       bf[n+3] <- ((BFB[n+3]+SKINV[i]*dilat)/(1.0+SKINC[i]*stric))*2.0^(error[n+3]/10.0)
       k <- floor(y[n+3]/5)
       pskin[i] <- P[k] + (P[k+1] - P[k])*(y[n+3]-5*k)/5.
       emax[i] <- (pskin[i]-P_air)*2.2*(H[i]-HR[i]*S[i])
       if (e[n+3]> emax[i]) {
         e[n+3] <- emax[i]
         }
       }

       # note Stolwijk's choice for 44.0; this might be P_H2O based on central blood temperature
       # or trunk core temperature
       e[5] = (pa["BASAL_METABOLIC_RATE"] + WORKI) * 0.0023 * (44.0 -P_air)
       # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION

       for (k in 1:24) {
         bc[k] = bf[k]*(y[k]-y[25])
         td[k] = TC[k]*(y[k]-y[k+1])
       }

       for (i in 1:6) {
         k <- 4*i - 3
         hf[k] <- qq[k] - e[k] - bc[k] - td[k]
         hf[k+1] <- qq[k+1] - bc[k+1] + td[k] - td[k+1]
         hf[k+2] <- qq[k+2] - bc[k+2] + td[k+1] - td[k+2]
         hf[k+3] <- qq[k+3] - bc[k+3] - e[k+3] + td[k+2] - H[i]*(y[k+3] - T_air)
       }

       hf[25] <- 0.
       for (k in 1:24) {
         hf[25] <- hf[25] + bc[k]
       }

    for (i in 1:6) {
     k = 4*i - 3

     # the rate of change of temperature is the total local heat flux divided by the thermal capacity
     ydot[k]   <- hf[k]/CC[k]
     ydot[k+1] <- hf[k+1]/CC[k+1]
     ydot[k+2] <- hf[k+2]/CC[k+2]
     ydot[k+3] <- hf[k+3]/CC[k+3]

     }
     # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION

     # rate of change of central blood temperature
     ydot[25] <- hf[25]/CC[25]

     # temporary, possibly inaccurate implementation of FF[k] value assignment. No consequences because
     # RATE coefficients are set to zero
     for(k in 1:25) {
         FF[k] <<- ydot[k] }

        # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION

    ewet <- 0.
    for (i in 1:6) {
       ewet <- ewet + (e[4*i]/emax[i])*S[i]/SA }
       # the fractional wetted area of the skin, pwet, is calculated in %
       pwet <- 100.*ewet


       co <- 0.
       hp <- 0.
       ev <- 0.
       tsk <- 0.
       tb <- 0.
       hflow <- 0.
       sbf <- 0.

       for (n in 1:24) {
          co <- co + bf[n]/60.
          hp <- hp + qq[n]
          ev <- ev + e[n]
          }

       for (i in 1:6) {
          sbf <- sbf + bf[4*i]/60.
          # variable named tsk instead of ts for skin temperature, because ts already has a different meaning in 'R'
          tsk <- tsk + y[4*i]*CC[4*i]/CCskin_tot
          }
    # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION
       for (n in 1:25) {
         # average body temperature
         tb <- tb + y[n]*CC[n]/CCbody_tot
         # net rate of heat storage for the whole body
         hflow <- hflow + hf[n]
         }

         ev <- ev/SA
         hp <- hp/SA
         hflow <- hflow/SA
         cond <- (hp - e[5]/SA -hflow)/(y[25]-tsk)

         # total mass of athlete plus bicycle
         # in function because the mass of the athlete can be made time dependent because of sweat loss
         totalM <- pa["Ma"]+pa["Mb"]


# here the non-thermal model variables are calculated
    # calculate distance y[26] covered by athlete given in meters
    ydot[26] <- y[27]

    # calculate speed of athlete
    # Pair, power used to overcome air resistance at present speed
    # E_Ra, force to overcome air resistance (N), is taken from Olds (1993)
    # this force is multiplied by road speed to determine instantaneous power spent to overcome air resistance
    # Slope <- pa["S"]
    Slope <- Slope_Trajectory(t,y,pa)

    # pa["RedAir"] takes into account reduction of air resistance above situation taken in Olds (1993) paper
    # coming from Di Prampero.
    # Pair is the power dissipated against air resistance increased by head wind
    Pair <- pa["RedAir"] * 0.19*(288/(T_air+273.15))*(SurfaceArea/1.77)*(v_cycle + v_wind)^2.* v_cycle
    if (pa["Air_res_red_by_alt"] == 1)  {
         Pair <- Pair * (Pbvar/755)
         }

    if ( (v_cycle + v_wind) >= 0.) {
        Pair <- Pair
        }
       else { Pair <- -Pair } # assuming that with net wind velocity blowing from behind extra power
                              # becomes available

    # the force to overcome rolling resistance (Olds, 1993) is multiplied by the ground speed
    atanSlope <- atan(Slope)

   if (phase == 1) {
    Prolling <- pa["Crr"] * cos(atanSlope)* totalM * v_cycle
    # the force to climb against gravity is multiplied by ground speed
    Fgrade <- totalM * pa["g"] * sin(atanSlope)
    Pgrade <- Fgrade * v_cycle

    # ACCELERATION
    # the available power for acceleration, Pacc in Watt, is calculated
    Pacc <- Pext - Pair - Prolling - Pgrade
    # Pacc can be constrained to provide forward thrust
    # if(Pacc <= 0.) {Pacc <- 0.; cat("\nWarning: Pacc was less than zero")}

    # the force for acceleration, Facc, is calculated
    # to avoid division by zero, giving infinite force, Facc is limited to 800 N, about 80 kg per leg,
    # at startup, v = 0 to 1 m/sec
    # if v gets higher, higher forces are possible reaching 1200 N at v = 5 m/s
    # THE FOLLOWING GROUP OF STATEMENTS MAY REQUIRE FUTURE IMPROVEMENT
      # warning message if speed becomes negative: a real cyclist would presumably dismount in that case
      # unless (s)he recovers very quickly
      if (v_cycle < 0.) cat("\nNegative speed, equals ", v_cycle," m/sec")
      # from the available power the force available for acceleration is calculated. This is the total force
      # minus the force for rolling resistance, air resistance and gravity
      # this force may be negative and lead to deceleration
      # the negative force leads to negative work (on the muscle) which is accompanied by increased
      # metabolism (eccentric work)
      # whether the latter effect on metabolism exists under these circumstances is unresolved
      # it has not yet been incorporated in the model

      # for all speeds different from zero, the available force for acceleration is calculated
      Facc <- 800. - Fgrade
      if (abs(v_cycle) <= 0.001) {Facc <- 800.-Fgrade} else {Facc <- Pacc / v_cycle}
      # the force for acceleration is limited by maximal values for the force which depend on the speed of the cyclist
      # this should perhaps be the rotation speed of the legs in a more accurate implementation
      if (v_cycle <= 1. && Facc > (800.-Fgrade)) Facc <- 800.-Fgrade
      if (v_cycle >= 1. && v_cycle < 5.) {
          Facc_max <- 800. + (1200. - 800.) * (v_cycle - 1.)/4. - Fgrade
          if (Facc > Facc_max) Facc <- Facc_max
          }
      if (v_cycle >= 5. && Facc > (1200.-Fgrade) ) Facc_max <- 1200. - Fgrade


      if (Pacc <0. && v_cycle < 0.) Facc <- -Pacc/v_cycle
    # the acceleration, acc, is calculated; it is taken into account that the model integrates in hours, while
    # the calculation in the power balance part gives acceleration in m / sec^2
    # y[27] is given in m / hr
    acc <- Facc / totalM
    }
    # before Phase I (exercise) no external work is done
    else {Prolling <- 0.; Pgrade <- 0.; Pair <- 0.; Pacc <- 0.; Facc <- 0.; acc <- 0.}

    # the acceleration in m/sec^2 is converted to m/hr^2 by multiplying by 3600^2 = 12960000  
    ydot[27] <- acc * 12960000

    # assemble a vector of function values and transport rates at this time point (=t) for output
    functionValuesOut <- c(     #first variables from the Stolwijk model are added
                                sweat=as.numeric(sweat),
                                emax_tr=as.numeric(emax[2]),
                                chill=as.numeric(chill),
                                co = as.numeric(co),
                                hp = as.numeric(hp),
                                ev = as.numeric(ev),
                                tskin = as.numeric(tsk),
                                tbody = as.numeric(tb),
                                hflow = as.numeric(hflow),
                                skinbf = as.numeric(sbf),
                                # speed, distance, altitude, slope grouped together
                                road_distance = as.numeric(y[26]),
                                speed = as.numeric(y[27]),
                                altitude = as.numeric(hAH),
                                slope = as.numeric(Slope),
                                # other variables
                                p_barometric = as.numeric(Pbvar),
                                Pair = as.numeric(Pair),
                                Prolling = as.numeric(Prolling),
                                Pgrade = as.numeric(Pgrade),
                                Pacc = as.numeric(Pacc),
                                Facc = as.numeric(Facc),
                                acc = as.numeric(acc),
                                # variables which are calculated
                                sum_Pext = as.numeric(Pair + Prolling + Pgrade + Pacc),
                                heat_production_Watt = as.numeric(hp*1.1622*SA),
                                sum_Watt = as.numeric(Pair + Prolling + Pgrade + Pacc + hp *1.1622*SA)
				)

     if(finish_time_flag == 0. && y[26]>=15500.) {
          finish_time_flag <<- 1.
          raw_finish_time <<- t
          raw_finish_distance <<- y[26]
          raw_finish_speed <<- y[27]
          }

     # INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION - INSIDE MODEL FUNCTION

    # print the time of the simulation to console to show progression of the simulation to the user
    # the 'timer' is updated after each time the value of simulation time is printed
    if ( (t - timer) > pa["plotinterval"] ) {
	cat("\ntime = ",round(t,3)," hr ","Distance = ",y[26],"m   Speed = ",y[27]/1000., "km/hr")
	# also the heat rate and Tair can be printed during progress, uncomment to print
        aux <- y[1]
	# the following statement adds next segment to the animation of the simulation
        lines(c(t_mem,t),c(y1_mem,aux),pch=20) # add point to the plot.
        # the first point of the next line segment to be added to the plot is stored here in t_mem and y1_mem
        # to be used the next time a point is added to the plot.
        # This point equals the last point of the last line segment.
        # The very first point of the runin phase and the first point of the second phase
        # is generated below, just before lsoda is called.
	t_mem <<- t
	y1_mem <<- aux
	flush.console()
	timer <<- t
    }

    # assemble the list that is returned by function 'model'. These are the time derivatives ydot and
    # calculated enzyme and transport rates which 'odesolve' uses to assemble reports
    list(ydot,functionValuesOut)
}
# Here FUNCTION "model" ENDS
# END FUNCTION MODEL - END FUNCTION MODEL - END FUNCTION MODEL - END FUNCTION MODEL - END FUNCTION MODEL


# SIMULATION SECTION - SPECIFIC SIMULATION DATA DIFFERENT FROM DEFAULT READ IN  FROM FILE

# read filename from small file
# this file should contain a line stating : filename <- "your_own_choice"
  source("filename.R")

# execution of the simulation

                flush.console() # works only under MS Windows - Comment out for other operating systems by adding a '#' at the very start of this line
		pa<-pa_default
		# setting default initial values by setting them equal to temperature set points 
		y_init <- c(TSET,x=0,v=0)

		# the names of the variables are put in an array of characters to be used with output.
                # The names are the same for array y.
                namesOutputVariables <- names(y_init)

                # Calculation of those parameters which need to be calculated


                # in the file read next parameters, initial values, TSET, and even the parameter
                # switch function can be adapted for each run, by overwriting their default values
		source(paste(filename,".R",sep=""))

		# body surface area according to Dubois formula, Eq. 5, Olds et al., 1993
		SurfaceArea<- pa["Ma"]^0.425 * pa["Ht"]^0.725*0.007184
		# the total surface area according to Stolwijk's model for the 74.1 kg man is almost equal to the
		# surface area calculated according to the Dubois formula.
		# surface areas of the segments are therefore not adjusted

		# basal metabolic rate is taken proportional to body weight
		pa["BASAL_METABOLIC_RATE"] <- pa["BASAL_METABOLIC_RATE"] * pa["Ma"]/74.1

                # CF_Swain, correction factor
		pa["RedAir"] <- (23.06 - 2.46*SurfaceArea)/18.7

                # plot axes for this particular iteration of the simulation run
                # if no animation is desired the next statement must be commented out.
                # y_axis_min and y_axis_max are numbers that can be set in filename.R
                plot(0,0,xlim=c(-pa["trunin"]+ pa["tbegin"],pa["tend"]),
                         ylim=c(pa["y_axis_min"],pa["y_axis_max"]),
                         xlab="Time (hours)", ylab="Temperature Brain (Degrees Celsius)",type="n")

		# 'timer' is reset to the first time point of the simulation run of this particular iteration
                timer <- pa["tbegin"]-pa["trunin"]

                # the runin_times vector is modified from default if desirable.
                # if no change is desired the next line can be commented out.
		runin_times <-seq(pa["tbegin"]-pa["trunin"],pa["tbegin"],pa["tstep"])


                # in the vector 'times' the desired time points for output
                # of the solution of the model integration are given.
		times <- seq(pa["tbegin"],pa["tend"],pa["tstep2"])


  # START OF THE RUNIN BLOCK
  # the following lines can be commented out if the start values for the second phase are given in a file.
  # Here the start results for the runin period can be generated by a runin period until a steady state
  # is reached.

  # generating the first point of the plot at very start runin of simulation
                t_mem <- timer
                y1_mem <- y_init[1]

                # hmax value is adapted to obtain sufficient accuracy for conditions with sudden, fast changes
                if (pa["Pulsatility"] > 0.) { hmax_value<-pa["hmax_Pulsatile"] }
                if (pa["Pulsatility"] <= 0.) { hmax_value<-pa["hmax_nonPulsatile"] }

# initial temperatures are read from directory on disk
                source("initial_values_vector.R")
                y_init <- as.numeric(initial_values)

# generating the first point of the plot at very start runin of simulation
                t_mem <- timer
                y1_mem <- y_init[1]

                runin_out <- as.data.frame(lsoda(y_init,runin_times, model, pa, 1e-6, 1e-6, hmax=hmax_value,
                              tcrit=pa["tbegin"]))
		last<-length(runin_out[,"time"])
                xinit <- as.numeric(runin_out[last,2:28])
# END OF THE RUNIN BLOCK

                # the race starts at t=0. with distance xinit[26] equal to 0
                # and speed xinit[27] also equal to 0
                xinit[26] <- 0.
                xinit[27] <- 0.

		names(xinit)<-namesOutputVariables

                # the timer is set to the begin of the simulation
		timer <- pa["tbegin"]

                # generating the first point for the plot during the second phase of simulation
                t_mem <- timer
                y1_mem <- runin_out[last,2]

		out <- as.data.frame(lsoda(xinit, times, model, pa,
		                            1e-6, 1e-6,hmax=hmax_value)) # hmax can be set to 0.0003 for accuracy

                 if (raw_finish_distance >= 15500.) {
                      finish_time1 <- raw_finish_time - (raw_finish_distance - 15500)/raw_finish_speed
                      fin_time1_min <- floor(finish_time1*60.)
                      fin_time1_sec <- (finish_time1*60.-fin_time1_min)*60.
                      cat("\nFinish time = ",fin_time1_min," min and ",fin_time1_sec," sec\n")
                    }else {
                      cat("\nNot enough simulation time to reach finish line\n\n")
                      finish_time_1 <- 0.
                      fin_time1_min <- 0.
                      fin_time1_sec <- 0.
                    }

                #save graphics
                currentGraphicsDevice <- dev.list()
                dev.print(postscript,paste("plot.",filename," ",date_aux,".ps",sep=""))

                # results are saved
                save.image(paste(filename," ",date_aux,".RData",sep=""))

                write.xlsx(x=out,file = paste("out",filename,"_",date_aux,".xlsx", sep=""),sheetName = "Results",
                                  append = FALSE, row.names = TRUE)

                #if the output to console should be saved: uncomment the next sink statement together with the one above
                #sink(file=NULL)