# Functions for R program Digital_Cyclist to switch parameter values during a simulation run
# developed by J.H.G.M. van Beek - May 2008 and December 2010, updated December 2013

#  This program simulates energy conversion and heat transport in a cyclist, in particular during a mountain time trial
#     Copyright (C) <2013>  <J.H.G.M. van Beek>
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.


# definition of functions which can make parameters during run variable
   Phase_switch <- function(t) {
     if (t<=0.) phase <- 0.
     if (t>0.)  phase <- 1.
     phase
     }

   Tair_switch <- function(t) {
     if (t < 0.5) {T_air <- 24.}
     if (t >= 0.5 && t < 2.5) {T_air <- 24.}
     if (t >= 2.5) {T_air <- 24.}
     T_air
   }

# here we declare a function which calculates the muscular power (work per unit time)
# available from the muscles of the athlete to sustain or change velocity
    # it is assumed that 1 ml of O2 (STPD) corresponds to 20.9 J/ml, corresponding to a respiratory quotient
    # of 0.96 (Di Prampero, 1979)
    # 1 kcal (th) = 4184 Joule
    # 1000 ml/min = 20,900 J/min = 4.99522 kcal/min = 299.713 kcal/hr

    # at t<0 WORK is calculated from VO2init i.e. the initial rate of oxygen consumption
    # the amount of WORK above basal metabolic rate of 74.4 kcal/hr may be due to muscle tension,
    # increased physiological activity of digestive system etc.
    # of this internal work nothing is used for external work Pext (Watt)
    # Pext equals WR in the Olds, 1993 paper
    Work_switch <- function(t, pa) {
      if (t<=0.) {
         # converting VO2init in litre O2 / min to WORK in kcal/hr
         WORK <- pa["VO2init"]* 299.713
         # calculating WORKI from oxygen uptake measurements before starting to cycle
         WORKI <- WORK - pa["BASAL_METABOLIC_RATE"]
         Pext <- 0.
         }
    # during exercise the work rate is set to a certain fraction of VO2max which can be sustained
    # for the duration of the exercise period.
    # see f factor in manuscripts by Olds, e.g. Fig. 1 in 1993 manuscript
      if (t>0.) {
         # nVO2 (normalized VO2max) =  0.4397485 + 0.0007373 * barometric pressure (mmHg)
         # here the barometric pressure calculated for that altitude is used : pa["Pbvar"]
         nVO2 =  0.4397485 + 0.0007373 * pa["Pbvar"]
         cat("\ntime = ",t," Pbar = ",pa["Pbvar"]," nVO2 = ",nVO2)
         # the maximum part of VO2max that can be used for a certain period of time
         partVO2 <- nVO2 * pa["brainpush"]* pa["VO2max"]
         # what is the energy turnover in kcal/hr that is equivalent to partVO2
         WORKEQ <- partVO2*299.713
         WORK <- WORKEQ
         # increase in energy turnover over basal metabolic rate in kcal/hr
         WORKEQI <- WORKEQ - pa["BASAL_METABOLIC_RATE"]
         # the part of increase energy turnover available for mechanical work done by the athlete
         Pext <- WORKEQI * pa["DeltaMechEff"]/0.8604201 # in Watt, calculated from Delta mechanical efficiency
         #Pext <- (partVO2 - pa["incpt"])/pa["grad"] # in Watt, according to the Olds (1993) paper
         # WORKI is the heat generation above baseline level associated with the muscle work performed
         WORKI <-  WORKEQI - Pext* 0.8604201 # converting 1 Watt to 0.8604201 kcal/hr
         }
      c(WORK, WORKI, Pext)
      }
